'use strict';

Object.defineProperty(exports, "__esModule", {
    value: true
});

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

exports.defaultShouldTransform = defaultShouldTransform;
exports.createAddTransformer = createAddTransformer;

/**
 * The default functions which decides if should transform the value.
 * @param {*} value
 * @param {string} key
 */
function defaultShouldTransform(value) {
    return typeof value !== 'undefined';
}

function createAddTransformer(shouldTransform, transformers, createTransform) {
    return function (prop, transform) {
        var transformer = {
            prop: prop,
            transform: transform
        };

        var newTransformers = [].concat(transformers, [transformer]);

        return createTransform(shouldTransform, newTransformers);
    };
}

function createTransform() {
    var shouldTransform = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : defaultShouldTransform;
    var transformers = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : [];

    var transformProperties = function transformProperties(obj) {
        return transformers.reduce(function (obj, transformer) {
            var prop = transformer.prop;
            var transform = transformer.transform;

            // Check if the property should be transformed
            if (!shouldTransform(obj[prop], prop)) {
                return obj;
            }

            var value = obj[prop];
            var newValue = transform(value, _extends({}, obj));

            var objWithNewValue = {};
            objWithNewValue[prop] = newValue;

            // Create new object with new
            return _extends({}, obj, objWithNewValue);
        }, _extends({}, obj));
    };

    transformProperties.add = createAddTransformer(shouldTransform, transformers, createTransform);

    // We expose the transformers for tests
    transformProperties._transformers = transformers;

    return transformProperties;
}

exports.default = createTransform;